##### crossing of trajectories

crossing.f = function(maxage = 10, n.tot.ind = 100, 
cor.u.v.chain = c(-0.5,0,0.5), seed.sim = c(20), psi.chain = c(0.5), beta_k0 = -0.96, Linf.common = 300,
 beta_t0 = -0.32, log_sigma_v = -1.5, log_sigma_u = -1.5, log_sigma_w = -4 ) {

#the dataset is Mark, length, age

##### plotting ####

size.label.x = 14
size.text.x = 9
line.lwd = 0.6
size.label.y = 14
size.text.y = 9
size.legend.text = 14
size.legend.title = 14
unit.legend.h = 1.4
unit.legend.w = 1.4
size.point = 3
col.line = "gray20"


###################



###### functions #######

########################


NormCorr.f <- function(n=100, rho = 0.4,sdfirst = 1) { 
     firstvect = rnorm(n,mean = 0, sd = sdfirst)
     secondvect = rnorm(n) 
     Z = cbind(firstvect, rho*firstvect+sqrt(1-rho^2)*secondvect) 
    return(Z) 
} 

#######################

VBsnov3_psi = function (beta,k0,log_sigma_v,v,log_sigma_u,u,t0,
log_sigma_w,w,plpo,psi.vb = 1,maxage=10) {

age = seq(1,maxage,1)
Length_age = rep(0,length(age))
gamma = exp(beta*10 + exp(log_sigma_v)*v)
k = exp(k0) * exp( exp(log_sigma_u)*u )
Linf = gamma 
t0 = t0 + exp(log_sigma_w)*w

for (ii in 1:length(age)) {

Length_age[ii] = Linf*(k^(psi.vb-1))*(1-exp(-k*(age[ii]-t0)))

}
if (plpo==0) {
plot(Length_age ~ age, xlim = c(0,max(age)),ylim = c(0,max(Length_age)),pch=16)
}

else if (plpo==1) {
points(Length_age ~ age,pch=15)
}

return(data.frame(Age=age,Length=Length_age))
}

##########################


cont.loop = 0

rep.row = length(psi.chain) * length(seed.sim) * length(cor.u.v.chain)

psi.cross.df = data.frame(psi = rep(0,rep.row),cor.u.v = rep(0,rep.row),
mean.age.cross = rep(0,rep.row),
norm.cross = rep(0,rep.row),seed = rep(0,rep.row), cv.length = rep(0,rep.row))

#### cycle for psi values

for (pcont in 1:length(psi.chain)) {

#### cycle for cor.u.v.chain values

 
for (cor.cont in 1:length(cor.u.v.chain)) {


#### cycle for seed.values

for(seed.cont in 1:length(seed.sim)) {

#### build the data set with input data #######


cont.loop = cont.loop + 1


beta_k0 = beta_k0 
beta = log(Linf.common * (exp(beta_k0))^(1-psi.vb))/10
beta_t0 = -0.32
log_sigma_v = log_sigma_v
log_sigma_u = log_sigma_u
log_sigma_w = log_sigma_w


n.tot.ind = n.tot.ind


mockrand = data.frame(Mark = (1:n.tot.ind),u = rep(0,n.tot.ind),w = rep(0,n.tot.ind),
v=rep(0,n.tot.ind))

set.seed(seed.sim[seed.cont])
uv = NormCorr.f(n.tot.ind,rho = cor.u.v.chain[cor.cont])
mockrand$u = uv[,1]
mockrand$v = uv[,2]
mockrand$w = rnorm(n.tot.ind,0,0)

uv.real.cor = cor.test(mockrand$u,mockrand$v)
uv.real.cor = round(uv.real.cor$estimate,digits = 2)


mockdataset = data.frame(Mark = rep(mockrand$Mark,each=maxage),
Length = rep(0,n.tot.ind*(maxage)),
Age = rep(0,n.tot.ind*(maxage)))
contlen = 1
for (j in 1:n.tot.ind) {
plot.mock =VBsnov3_psi(beta,beta_k0,log_sigma_v,mockrand[j,"v"],log_sigma_u,
mockrand[j,"u"],beta_t0, log_sigma_w,mockrand[j,"w"], 2,psi.vb = psi.chain[pcont],
 maxage=maxage)

#mockdataset$Length[contlen: ((maxage+1)*i)] = plot.mock$Length
mockdataset$Length[(sum(mockdataset$Length>0)+1): ((maxage)*j)] = plot.mock$Length

#mockdataset$Age[contlen: ((maxage+1)*i)] = plot.mock$Age

mockdataset$Age[(sum(mockdataset$Age>0)+1): ((maxage)*j)] = plot.mock$Age


contlen = length(mockdataset$Length[mockdataset$Length>0])+1
}


##########################################

#### Test crossing of trajectories


cont.cross = 0

unique.mark = unique(mockdataset$Mark)

cross.df = data.frame(Age = 1:maxage, n.cross = rep(0,maxage)) 

for (mark.cont in 1:length(unique.mark)) {

print(mark.cont)

length.target = mockdataset$Length[mockdataset$Mark == unique.mark[mark.cont]]

for (mark.test in 1:length(unique.mark)) {

if (unique.mark[mark.cont]!= unique.mark[mark.test]) {

length.tested =  mockdataset$Length[mockdataset$Mark == unique.mark[mark.test]]

length.confr = ifelse(length.target<=length.tested,-1,1)

#print(length.confr)

age.cross = which(length.confr!=length.confr[1])[1]

#print(age.cross)

if(!is.na(age.cross)) {
cross.df[age.cross,"n.cross"] = cross.df[age.cross,"n.cross"] + 1
cont.cross = cont.cross + 1
#print(cont.cross)
}
}
}
}

gcross <- ggplot(mockdataset, 
aes(x = Age, y = Length))

gcross <- gcross + geom_line(aes(group = Mark),col="black",lwd = line.lwd) +
      theme(plot.title = element_text(lineheight=.8, face="bold",size = 25),
    plot.background = element_blank()
   ,panel.grid.major = element_blank()
   ,panel.grid.minor = element_blank()
   ,panel.border = element_blank()
   ,panel.background = element_blank()) +theme(axis.line = element_line(color = 'black')) +
      scale_y_continuous(breaks=c(0,100,200,300,400,500),
      limits=c(0,max(mockdataset$Length))) +
	  scale_x_continuous(breaks=0:maxage,limits=c(0, maxage)) +
	   theme(
	   axis.title.x = element_text(size=size.label.x,vjust=0),
           axis.text.x  = element_text(size=size.text.x),
           axis.title.y = element_text(size=size.label.y, vjust = 0.1),
           axis.text.y  = element_text(size=size.text.y)) +
           labs(x="Age", y ="Length") #+
           #annotate("text", x = 1, y = 400, label = "a") +
           #annotate("text", x = 4, y = 50, label = 
           #bquote(.(mockrep.postmode.neg))) +
           #annotate("text", x = 4, y = 70, label = 
           #cor.lb.neg, parse = T) +
           #bquote("italic(r) = ",.(uv.real.cor))) + 
           #annotate("text", x = 1, y = 450, label = lb.neg,parse = TRUE)

	print(gcross)


sd.length = with(subset(mockdataset,Age==maxage),sd(Length,na.rm = T))
mean.length = with(subset(mockdataset,Age==maxage),mean(Length,na.rm = T))
cv.length = sd.length/mean.length


cross.df[,"n.cross"] = cross.df[,"n.cross"]/2 
cont.cross = cont.cross/2

print(cont.cross)
norm.cont.cross = cont.cross/n.tot.ind
print(norm.cont.cross)
mean.age.cross = weighted.mean(cross.df$Age,cross.df$n.cross,na.rm = T)

psi.cross.df$psi[cont.loop] = psi.chain[pcont]
psi.cross.df$cor.u.v[cont.loop] = cor.u.v.chain[cor.cont]
psi.cross.df$mean.age.cross[cont.loop] = mean.age.cross
psi.cross.df$norm.cross[cont.loop] = norm.cont.cross
psi.cross.df$seed[cont.loop] = seed.sim[seed.cont]
psi.cross.df$cv.length[cont.loop] = cv.length

}

}

}

#return(cross.df)
#return(cont.cross)
return(psi.cross.df)

}






