# Sets up the data in R

library(R2admb)

## create matrix model depending on the input in the global env


### The if below set up the formulas for linf and k depending on the environmental variable used

if (linf_var == "Density") {
linf.formula =  ~ Density_first_year}

if (linf_var == "Cohort") {
	linf.formula =  ~ Cohort
	}

if (linf_var == "Const") {
		linf.formula =  ~ 1
		}
		
if (k_var == "Density") {
k.formula =  ~ Density_first_year
}

if (k_var == "Cohort") {
	k.formula =  ~ Cohort
	}

if (k_var == "Const") {
		k.formula =  ~ 1
		}
		
		
#t0 is always at the population level

t0.formula = ~ 1

# build the matrices

linf.mm = model.matrix(linf.formula,data=data_growth)
k.mm = model.matrix(k.formula,data=data_growth)
t0.mm = model.matrix(t0.formula,data = data_growth)



## Density is not used in this work, but I keep it here anyway for possible future use

X = linf.mm

if (linf_var == "Density") {
X[,2]=X[,2]/1000
}

X_k0 = k.mm

if (k_var =="Density") {
X_k0 = k.mm
}
X_t0 = t0.mm

#X=X[,c(1,20)]
#X_k0=as.matrix(X_k0[,1])

#####
## ordering Marks (tag of individuals)

diffmark  = diff(data_growth$Mark)
diffmark  = ifelse(diffmark==0,0,1)

##### vectors of the values of psi (from 0 to 1 with step 0.1)

psi_fixed.vett = seq(0,1,0.1)# vectors of psi values to be fitted/tested

#matrix of AIC values for the models
AIC.vett = matrix(0,length(psi_fixed.vett),2)# create an empty matrix for later storing of AIC model values


## list for collection of model fitting results
synth.list.13 = vector("list", length(psi_fixed.vett))# list for storing results


## looping over the values of psi
for (i in 1:length(psi_fixed.vett)) {# loop for every value of psi

#define value of psi
psi_fixed = psi_fixed.vett[i]# takes value of psi

### writes the dat file - remember that write_dat is in ADMButil.s

write_dat("m_grow13",list(n=nrow(data_growth),
		length=data_growth$Length,
		age=data_growth$Age,
                total_marks=length(unique(data_growth$Mark)),
		start=c(1,which(diffmark>0)+1),
		stop=c(which(diffmark>0),nrow(data_growth)),
		q=ncol(X),
		X=X,
		q_k0=ncol(X_k0),
		X_k0=X_k0,
		q_t0=ncol(X_t0),
X_t0=X_t0,psi_fixed = psi_fixed,
check=12345))




system("./m_grow13 -est") ### calls m_grow18 with no standard deviations computed. To compute standard deviations, delete -est

results.par = par_read("m_grow13.par") # reads the parameter estimates and other quantities (number of parameters, gradient given by ADMB etc.)

assign(paste("results.par.zak0.13psi.",psi_fixed.vett[i],sep=""), results.par)## assigns a name to the model

pred = read.table("m_grow13.rep")# .rep contains the predictions (using the model) of data used for fitting the model

assign(paste("results.rep.zak0.13psi.",psi_fixed.vett[i],sep=""), pred)#assigns a name to model predictions

prov.AIC = -2*results.par$n_par - 2*results.par$loglik #compute AIC (the change of sign is due to a bug in ADMButils.s)
AIC.vett[i,1] = psi_fixed  ## value of psi
AIC.vett[i,2] = prov.AIC # value of AIC
prov.gradient = results.par$gradient

assign(paste("results.AIC.zak0.13psi.",psi_fixed.vett[i],sep=""), prov.AIC)  ##saves the value of AIC for the model

assign(paste("results.grad.zak0.13psi.",psi_fixed.vett[i],sep=""), prov.gradient) # saves the value of ADMB gradiet

# here below I assign to the list synth.list.18 the output of the model fitting

synth.list.13[[i]]$name = paste("results.par.zak0.13psi.",psi_fixed.vett[i],sep="") #name of the model
synth.list.13[[i]]$AIC = prov.AIC #AIC value
synth.list.13[[i]]$par = results.par ## parameter estimates
synth.list.13[[i]]$rep = pred # model predictions (hindcasting)
synth.list.13[[i]]$grad = prov.gradient # gradient of ADMB for the model
}

best.psi = AIC.vett[which(AIC.vett[,2] == (min(AIC.vett[,2]))),1]  # psi of the best model (the one with the lowest AIC)

